/* $Id: cfg.c,v 1.11 2003/01/24 18:47:21 swaj Exp $ */

/*
 * Was written by Alexey Semenov (c) 2001-2003
 * See COPYING for copying policy
 */


#define __CFG_C_
#include "cfg.h"
#undef __CFG_C_
#include "db_sql.h"


Cfg* cfg_read(void)
{
	gchar cf[256];
	gchar *ptr;

	memset(&my_cfg, 0, sizeof(Cfg));

	strncpy (my_cfg.host, "localhost", sizeof (my_cfg.host)-1); 
	strncpy (my_cfg.dicdb, DEFAULT_DICDB, sizeof (my_cfg.dicdb)-1);
	strncpy (my_cfg.font_lat, DEFAULT_FONT, sizeof (my_cfg.font_lat)-1);
	strncpy (my_cfg.font_latb, DEFAULT_FONT, sizeof (my_cfg.font_latb)-1);
	strncpy (my_cfg.font_rus, DEFAULT_FONT, sizeof (my_cfg.font_rus)-1);
	strncpy (my_cfg.font_rusb, DEFAULT_FONT, sizeof (my_cfg.font_rusb)-1);
	strncpy (my_cfg.bg_color_sel, BG_COLOR_SEL_DEFAULT,
		 sizeof (my_cfg.bg_color_sel)-1);
	strncpy (my_cfg.geometry, DEFAULT_GEOMETRY, sizeof (my_cfg.geometry)-1);
	cfg_convert_geometry(&my_cfg.geom, my_cfg.geometry);

	my_cfg.w_search_mode = W_SEARCH_MODE_DEF;
	my_cfg.a_search_mode = A_SEARCH_MODE_DEF;
	my_cfg.max_search = MAX_SEARCH_DEF;
	my_cfg.clear_new_search = CLEAR_NEW_SEARCH_DEF;

	g_snprintf(cf, sizeof(cf), "%s", GLOB_CONF_FILE);
	LOG("reading glob rc file: %s", cf);
	cfg_read_v(cf);

	g_snprintf(cf, sizeof(cf), "%s%s", g_get_home_dir(), CONF_FILE);
	LOG("reading user rc file: %s", cf);
	if (cfg_read_v(cf)) my_cfg.type = 1;

	LOG("reading env vars");

	if ((ptr=getenv(DICTIONARY_HOST)) != NULL) {
		LOG("%s = %s",DICTIONARY_HOST, ptr);
		strncpy (my_cfg.host, ptr, sizeof (my_cfg.host)-1);
	} 

	if ((ptr=getenv(DICTIONARY_DB)) != NULL) {
		LOG("%s = %s",DICTIONARY_DB, ptr);
		strncpy (my_cfg.host, ptr, sizeof (my_cfg.host)-1);
	} 

	if ((ptr=getenv(DICTIONARY_USER)) != NULL) {
		LOG("%s = %s",DICTIONARY_USER, ptr);
		strncpy (my_cfg.host, ptr, sizeof (my_cfg.host)-1);
	} 

	if ((ptr=getenv ( DICTIONARY_PASSWD ) ) != NULL) {
		LOG("%s = %s",DICTIONARY_PASSWD, ptr);
		strncpy (my_cfg.host, ptr, sizeof (my_cfg.host)-1);
	} 

	return &my_cfg;
}


static
Cfg* cfg_read_v(gchar *fn)
{
	gchar cf[256];
	gchar *ptr1, *ptr2, *ptr3;
	FILE *fl;

	g_snprintf(cf, sizeof(cf), "%s", fn);
	LOG("reading configuration file: %s", cf);

	if ((fl = fopen(cf,"r")) == NULL) {
		fprintf(stderr, "cannot open %s: %s\n", cf, g_strerror (errno));
		LOG("reading configuration failed: %s", g_strerror(errno));
		return NULL; /* &my_cfg; */
	}
	while ((fgets(cf, sizeof(cf), fl))!=NULL) {
		cf[strlen(cf)-1] = 0;	/* chomp */
		if (strchr(cf, DELIM) == NULL) continue;
		if ((ptr3 = strchr(cf, REM)) != NULL) *ptr3 = 0;
                if (strlen(cf) == 0) continue;

		cfg_split_string (cf, &ptr1, &ptr2);

                if ((ptr1 != NULL) && (ptr2 != NULL)) {

			if (strcmp (ptr1, "host") == 0) {
				strncpy (my_cfg.host, ptr2, sizeof (my_cfg.host)-1);
				continue;
                        }
			if (strcmp (ptr1, "dicdb") == 0) {
				strncpy (my_cfg.dicdb, ptr2, sizeof (my_cfg.dicdb)-1);
				continue;
                        }
			if (strcmp (ptr1, "user") == 0) {
				strncpy (my_cfg.user, ptr2, sizeof (my_cfg.user)-1);
				continue;
                        }
			if (strcmp (ptr1, "password") == 0) {
				strncpy (my_cfg.passwd, ptr2, sizeof (my_cfg.passwd)-1);
				continue;
                        }
			if (strcmp (ptr1, "font_lat") == 0) {
				strncpy (my_cfg.font_lat, ptr2, sizeof (my_cfg.font_lat)-1);
				continue;
                        }
			if (strcmp (ptr1, "font_latb") == 0) {
				strncpy (my_cfg.font_latb, ptr2, sizeof (my_cfg.font_latb)-1);
				continue;
                        }
			if (strcmp (ptr1, "font_rus") == 0) {
				strncpy (my_cfg.font_rus, ptr2, sizeof (my_cfg.font_rus)-1);
				continue;
                        }
			if (strcmp (ptr1, "font_rusb") == 0) {
				strncpy (my_cfg.font_rusb, ptr2, sizeof (my_cfg.font_rusb)-1);
				continue;
                        }
			if (strcmp (ptr1, "font_rusb") == 0) {
				strncpy (my_cfg.font_rusb, ptr2, sizeof (my_cfg.font_rusb)-1);
				continue;
                        }
			if (strcmp (ptr1, "latex_preamble") == 0) {
				cfg_conv_nline(ptr2);
				strncpy (my_cfg.latex_preamble, ptr2, sizeof (my_cfg.latex_preamble)-1);
				continue;
                        }
#ifdef USE_AUDIO
			if (strcmp (ptr1, "speech_program") == 0) {
				strncpy (my_cfg.speech_prg, ptr2, sizeof (my_cfg.speech_prg)-1);
				continue;
                        }
#endif /* USE_AUDIO */
			if (strcmp (ptr1, "geometry") == 0) {
				strncpy (my_cfg.geometry, ptr2, sizeof (my_cfg.geometry)-1);
				continue;
                        }
			if (strcmp (ptr1, "dictionary") == 0) {
				strncpy (my_cfg.cur_dic, ptr2, sizeof (my_cfg.cur_dic)-1);
				continue;
                        }

			if (strcmp (ptr1, BG_COLOR_SEL) == 0) {
				strncpy (my_cfg.bg_color_sel,
					 ptr2, sizeof (my_cfg.bg_color_sel)-1);
				continue;
                        }

			if (strcmp (ptr1, W_SEARCH_MODE) == 0) {
				my_cfg.w_search_mode = strtoul(ptr2, NULL, 0);
				if (my_cfg.w_search_mode > W_SEARCH_MODE_MAX)
					my_cfg.w_search_mode = W_SEARCH_MODE_DEF;
				continue;
                        }

			if (strcmp (ptr1, A_SEARCH_MODE) == 0) {
				my_cfg.a_search_mode = strtoul(ptr2, NULL, 0);
				if (my_cfg.a_search_mode > A_SEARCH_MODE_MAX)
					my_cfg.a_search_mode = A_SEARCH_MODE_DEF;
				continue;
                        }

			if (strcmp (ptr1, MAX_SEARCH) == 0) {
				my_cfg.max_search = strtoul(ptr2, NULL, 0);
				if (my_cfg.max_search == 0)
					my_cfg.max_search = MAX_SEARCH_DEF;
				continue;
                        }

			if (strcmp (ptr1, CLEAR_NEW_SEARCH) == 0) {
				my_cfg.clear_new_search = strtoul(ptr2, NULL, 0);
				if (my_cfg.clear_new_search > 1)
					my_cfg.clear_new_search = CLEAR_NEW_SEARCH_DEF;
				continue;
                        }
		}
	}
	fclose (fl);

	cfg_convert_geometry(&my_cfg.geom, my_cfg.geometry);

	LOG("reading configuration succesful");
	my_cfg.parse_ok = 1;
	return &my_cfg;
}


static
void cfg_conv_nline (gchar *s)
{
	while ((s=strstr (s, NEWLINE)) != NULL) {
		memcpy (s, " \n", 2); 
	}
}


static
void cfg_convert_geometry (Geometry* g, gchar* gt)
{
	gchar* ptr;

	if ((ptr = strtok (gt, "x \n")) == NULL) {
		LOG("NULL when parsing w");
		return;
	} else {
		g->w = g_strtod (ptr, NULL);
	}
	if ((ptr = strtok (NULL, "+ \n")) == NULL) {
		LOG("NULL when parsing h");
		return;
	} else {
		g->h = g_strtod (ptr, NULL);
	}
	if ((ptr = strtok (NULL, "+ \n")) == NULL) {
		LOG("NULL when parsing x");
		return;
	} else {
		g->x = g_strtod (ptr, NULL);
	}

	if ((ptr = strtok (NULL, "+ \n")) == NULL) {
		LOG("NULL when parsing y");
		return;
	} else {
		g->y = g_strtod (ptr, NULL);
	}
}


void cfg_save(void)
{
	gchar cf[256], cf2[256], buf[512];
	gint host_s, user_s, passwd_s, font_lat_s, font_latb_s, font_rus_s, font_rusb_s, geometry_s, dictionary_s;
	gint bg_color_sel_s, w_search_mode_s, a_search_mode_s, max_search_s, clear_new_search_s;
	FILE *fl1, *fl2;

	LOG("saving cfg...");
	host_s = user_s = passwd_s = font_lat_s = font_latb_s = font_rus_s = font_rusb_s = geometry_s = dictionary_s = 0;
	bg_color_sel_s = w_search_mode_s = a_search_mode_s = max_search_s = clear_new_search_s =0;

	g_snprintf(cf, sizeof(cf), "%s%s", g_get_home_dir(), CONF_FILE);

	if ((fl1 = fopen(cf,"r")) == NULL) {
		fprintf(stderr, "cannot open %s: %s\n", cf, g_strerror (errno));
		LOG("saving configuration failed: %s", g_strerror(errno));
		return;
	}

	g_snprintf(cf2, sizeof(cf2), "%s%s.tmp", g_get_home_dir(), CONF_FILE);

	if ((fl2 = fopen(cf2,"w+")) == NULL) {
		fprintf(stderr, "cannot open %s: %s\n", cf, g_strerror (errno));
		LOG("saving configuration failed: %s", g_strerror(errno));
		fclose(fl1);
		return;
	}

	while ((fgets(buf, sizeof(buf), fl1))!=NULL) {

		if ((buf[0] != REM ) && (strlen(buf) >0)) {

			if (strncmp (buf, HOST, strlen(HOST)) == 0) {
				g_snprintf (buf, sizeof(buf), "host = %s\n", my_cfg.host);
				host_s = 1;
			}
			if (strncmp (buf, DICDB, strlen(HOST)) == 0) {
				g_snprintf (buf, sizeof(buf), "dicdb = %s\n", my_cfg.dicdb);
				host_s = 1;
			}
			else if (strncmp (buf, USER, strlen(USER)) == 0) {
				g_snprintf (buf, sizeof(buf), "user = %s\n", my_cfg.user);
				user_s = 1;
			}
			else if (strncmp (buf, PASSWD, strlen(PASSWD)) == 0) {
				g_snprintf (buf, sizeof(buf), "passwd = %s\n", my_cfg.passwd);
				passwd_s = 1;
			}
			else if (strncmp (buf, FONT_LAT, strlen(FONT_LAT)) == 0) {
				g_snprintf (buf, sizeof(buf), "font_lat = %s\n", my_cfg.font_lat);
				font_lat_s = 1;
			}
			else if (strncmp (buf, FONT_LATB, strlen(FONT_LATB)) == 0) {
				g_snprintf (buf, sizeof(buf), "font_latb = %s\n", my_cfg.font_latb);
				font_latb_s = 1;
			}
			else if (strncmp (buf, FONT_RUS, strlen(FONT_RUS)) == 0) {
				g_snprintf (buf, sizeof(buf), "font_rus = %s\n", my_cfg.font_rus);
				font_rus_s = 1;
			}
			else if (strncmp (buf, FONT_RUSB, strlen(FONT_RUSB)) == 0) {
				g_snprintf (buf, sizeof(buf), "font_rusb = %s\n", my_cfg.font_rusb);
				font_rusb_s = 1;
			}
			else if (strncmp (buf, DICTIONARY, strlen(DICTIONARY)) == 0) {
				g_snprintf (buf, sizeof(buf), "dictionary = %s\n", my_cfg.cur_dic);
				dictionary_s = 1;
			}

			else if (strncmp (buf, BG_COLOR_SEL" ", strlen(BG_COLOR_SEL" ")) == 0) {
				g_snprintf (buf, sizeof(buf), "%s = %s\n", BG_COLOR_SEL,  my_cfg.bg_color_sel);
				bg_color_sel_s = 1;
			}

			else if (strncmp (buf, W_SEARCH_MODE" ", strlen(W_SEARCH_MODE" ")) == 0) {
				g_snprintf (buf, sizeof(buf), "%s = %d\n", W_SEARCH_MODE,  my_cfg.w_search_mode);
				w_search_mode_s = 1;
			}

			else if (strncmp (buf, A_SEARCH_MODE" ", strlen(A_SEARCH_MODE" ")) == 0) {
				g_snprintf (buf, sizeof(buf), "%s = %d\n", A_SEARCH_MODE,  my_cfg.a_search_mode);
				a_search_mode_s = 1;
			}

			else if (strncmp (buf, MAX_SEARCH" ", strlen(MAX_SEARCH" ")) == 0) {
				g_snprintf (buf, sizeof(buf), "%s = %d\n", MAX_SEARCH,  my_cfg.max_search);
				max_search_s = 1;
			}

			else if (strncmp (buf, CLEAR_NEW_SEARCH" ", strlen(CLEAR_NEW_SEARCH" ")) == 0) {
				g_snprintf (buf, sizeof(buf), "%s = %d\n", CLEAR_NEW_SEARCH,  my_cfg.clear_new_search);
				clear_new_search_s = 1;
			}

			else if (strncmp (buf, GEOMETRY, strlen(GEOMETRY)) == 0) {
				g_snprintf (buf, sizeof(buf), "geometry = %dx%d+%d+%d\n", my_cfg.geom.w, my_cfg.geom.h, my_cfg.geom.x, my_cfg.geom.y);
				geometry_s = 1;
			}
			fprintf (fl2, "%s", buf);

		} else {
			fprintf (fl2, "%s", buf);
		}
	}	
	fclose (fl1);

	if (!dictionary_s ) {
		g_snprintf (buf, sizeof(buf), "dictionary = %s\n", my_cfg.cur_dic);
		fprintf (fl2, "%s", buf);
	}
	if (!geometry_s) {
		g_snprintf (buf, sizeof(buf), "geometry = %dx%d+%d+%d\n", my_cfg.geom.w, my_cfg.geom.h, my_cfg.geom.x, my_cfg.geom.y);
		fprintf (fl2, "%s", buf);
	}
	if (!font_lat_s) {
		g_snprintf (buf, sizeof(buf), "font_lat = %s\n", my_cfg.font_lat);
		fprintf (fl2, "%s", buf);
	}
	if (!font_latb_s) {
		g_snprintf (buf, sizeof(buf), "font_latb = %s\n", my_cfg.font_latb);
		fprintf (fl2, "%s", buf);
	}
	if (!font_rus_s) {
		g_snprintf (buf, sizeof(buf), "font_rus = %s\n", my_cfg.font_rus);
		fprintf (fl2, "%s", buf);
	}
	if (!font_rusb_s) {
		g_snprintf (buf, sizeof(buf), "font_rusb = %s\n", my_cfg.font_rusb);
		fprintf (fl2, "%s", buf);
	}

	if (!bg_color_sel_s) {
		g_snprintf (buf, sizeof(buf), "%s = %s\n", BG_COLOR_SEL,  my_cfg.bg_color_sel);
		fprintf (fl2, "%s", buf);
	}

	if (!w_search_mode_s) {
		g_snprintf (buf, sizeof(buf), "%s = %d\n", W_SEARCH_MODE,  my_cfg.w_search_mode);
		fprintf (fl2, "%s", buf);
	}

	if (!a_search_mode_s) {
		g_snprintf (buf, sizeof(buf), "%s = %d\n", A_SEARCH_MODE,  my_cfg.a_search_mode);
		fprintf (fl2, "%s", buf);
	}

	if (!max_search_s) {
		g_snprintf (buf, sizeof(buf), "%s = %d\n", MAX_SEARCH,  my_cfg.max_search);
		fprintf (fl2, "%s", buf);
	}

	if (!clear_new_search_s) {
		g_snprintf (buf, sizeof(buf), "%s = %d\n", CLEAR_NEW_SEARCH,  my_cfg.clear_new_search);
		fprintf (fl2, "%s", buf);
	}

	fclose (fl2);
	unlink (cf);
	rename (cf2, cf);
}


static
void cfg_split_string (gchar *str, gchar **ptr1, gchar **ptr2)
{
	gchar *p1 = *ptr1;
	gchar *p3, *p4;

	*ptr1 = str;
	*ptr2 = NULL;

	if ((p3 = strchr(str, DELIM)) == NULL) {
		return;
	}
	for (p4 = p3-1; p4 > p1; p4-- ) {
		if ( *p4 != ' ') {
			break;
		} else {
			*p4 = 0x0;
		}
	}
	p3++;
	for (p4 = p3; p4 < p3+strlen(p3); p4++) {
		if ((*p4 != ' ') && (*p4 != '\t'))  {
			break;
		}
	}
	*ptr2 = p4;

	for (p3 = p4+strlen(p4)-1; p3 > p4; p3--) {

		if ((*p3 != ' ') && (*p3 != '\t') && (*p3 != '=')   )  {
			break;
		} else {
			*p3 = 0x0;
		}
        }
}


/* __END__ */
