/* $Id: cfg.h,v 1.11 2003/01/24 18:47:21 swaj Exp $ */

/*
 * Was written by Alexey Semenov (c) 2001-2003
 * See COPYING for copying policy
 */


#ifndef __CFG_H_
#define __CFG_H_ 1


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <glib.h>

#include "logging.h"
#include "defs.h"


typedef struct geo {
		gint w;
		gint h;
		gint x;
		gint y;
} Geometry;


typedef struct cfg {
	gchar host[64];
	gchar dicdb[64];
	gchar user[32];
	gchar passwd[32];
	gchar font_lat[128];
	gchar font_latb[128];
	gchar font_rus[128];
	gchar font_rusb[128];
	gchar latex_preamble[512];
	gchar geometry[32];
	gchar cur_dic[64];
	gchar bg_color_sel[32];

	gint  w_search_mode;
	gint  a_search_mode;
	gint  max_search;
        gint  clear_new_search;

#ifdef USE_AUDIO
	gchar speech_prg[256];
#endif /* USE_AUDIO */

	Geometry geom;
	gint parse_ok;

	gint type;

#ifdef USE_SOCKET
	gchar sock_lsn[LOCAL_SOCK_SIZE];
#endif /* USE_SOCKET */

} Cfg;


#ifndef WORK_DIR
#define WORK_DIR G_DIR_SEPARATOR_S".gtkdic"
#endif


Cfg* cfg_read(void);
void cfg_save(void);



#ifndef DEFAULT_GEOMETRY
#define DEFAULT_GEOMETRY "800x600+0+0"

#define DEFAULT_GEOMETRY_H 600
#define DEFAULT_GEOMETRY_W 800
#define DEFAULT_GEOMETRY_X 0
#define DEFAULT_GEOMETRY_Y 0

#define DEFAULT_GEOMETRY_MAX_H 4000
#define DEFAULT_GEOMETRY_MAX_W 6000
#define DEFAULT_GEOMETRY_MAX_X 25000
#define DEFAULT_GEOMETRY_MAX_Y 25000

#endif /* DEFAULT_GEOMETRY */



#ifdef  __CFG_C_ /* Private declarations */

static Cfg my_cfg;
static Cfg* cfg_read_v(gchar*);
static void cfg_conv_nline(gchar*);
static void cfg_convert_geometry (Geometry*, gchar*);
static void cfg_split_string (gchar*, gchar**, gchar**);


#ifndef DEFAULT_FONT
#define DEFAULT_FONT "fixed"
#endif

#ifndef DEFAULT_DICDB
#define DEFAULT_DICDB "ptkdic"
#endif

#ifndef CONF_FILE
#define CONF_FILE G_DIR_SEPARATOR_S".gtkdic"G_DIR_SEPARATOR_S"gtkdicrc"
#endif

#ifndef GLOB_CONF_FILE
#define GLOB_CONF_FILE G_DIR_SEPARATOR_S"etc"G_DIR_SEPARATOR_S"gtkdicrc"
#endif 

/* env settings */
#ifndef DICTIONARY_HOST
#define DICTIONARY_HOST "DIC_HOST"
#endif

#ifndef DICTIONARY_DB
#define DICTIONARY_DB "DIC_DB"
#endif

#ifndef DICTIONARY_USER
#define DICTIONARY_USER "DIC_USER"
#endif

#ifndef DICTIONARY_PASSWD
#define DICTIONARY_PASSWD "DIC_PWD"
#endif

#ifndef BG_COLOR_SEL_DEFAULT
#define BG_COLOR_SEL_DEFAULT "yellow"
#endif


#define DELIM '='
#define REM '#'
#define NEWLINE "\\n"

#define HOST              "host "
#define DICDB             "dicdb "
#define USER              "user "
#define PASSWD            "passwd "
#define FONT_LAT          "font_lat "
#define FONT_LATB         "font_latb "
#define FONT_RUS          "font_rus "
#define FONT_RUSB         "font_rusb "
#define GEOMETRY          "geometry "
#define DICTIONARY        "dictionary "

#define BG_COLOR_SEL      "bg_color_sel"
#define W_SEARCH_MODE     "word_search_mode"
#define	A_SEARCH_MODE     "art_search_mode"
#define	MAX_SEARCH        "max_search"
#define CLEAR_NEW_SEARCH  "clear_new_search"


#ifndef W_SEARCH_MODE_DEF
#define W_SEARCH_MODE_DEF S_WORD_NORM_GLOB
#define W_SEARCH_MODE_MAX 4
#endif

#ifndef A_SEARCH_MODE_DEF
#define A_SEARCH_MODE_DEF S_ART_BOUNDARY_BOTH
#define A_SEARCH_MODE_MAX 8
#endif

#ifndef MAX_SEARCH_DEF
#define MAX_SEARCH_DEF 120
#endif


#ifndef CLEAR_NEW_SEARCH_DEF
#define CLEAR_NEW_SEARCH_DEF 1
#endif


#endif /* __CFG_C_ */


#endif /* __CFG_H_ */


/* __END__ */
