/* $Id: gtkdicctl.c,v 1.7 2003/01/24 18:47:21 swaj Exp $ */

/*
 * Was written by Alexey Semenov (c) 2001-2003
 * See COPYING for copying policy
 */


#define __GTKDICCTL_C_ 1
#include "gtkdicctl.h"
#undef __GTKDICCTL_C_


int main (int argc, char **argv)
{
	char sw;

	if (argc < 3) {
		print_about();
		exit (1);
	}

	while ( (sw = getopt(argc, argv, "c:")) != -1) {
		switch (sw) {
		case 'c':
			if (strcmp (optarg, S_CMD_CLIPBOARD_SEARCH_WORD)==0) {
				LOG("%s", S_CMD_CLIPBOARD_SEARCH_WORD);
				uniconify();
				my_msg.cmd = CMD_CLIPBOARD_SEARCH_WORD;

			} else if (strcmp (optarg, S_CMD_CLIPBOARD_SEARCH_ART)==0) {
				LOG("%s", S_CMD_CLIPBOARD_SEARCH_ART);
				uniconify();
				my_msg.cmd = CMD_CLIPBOARD_SEARCH_ART;

			} else if (strcmp (optarg, S_CMD_QUIT_NO_SAVE) ==0) {
				LOG("%s", S_CMD_QUIT_NO_SAVE);
				my_msg.cmd = CMD_QUIT_NO_SAVE;

			} else if (strcmp (optarg, S_CMD_ICONIFY)==0) {
				LOG("%s", S_CMD_ICONIFY);
				my_msg.cmd = CMD_ICONIFY;

			} else if (strcmp (optarg, S_CMD_UNICONIFY)==0) {
				LOG("%s", S_CMD_UNICONIFY);
				my_msg.cmd = CMD_UNICONIFY;

			}  else {
				LOG("unknown: %s", optarg);
				print_about();
				exit (1);
			}
			if (!send_cmd()) exit (-1);
			break;
		default:
			exit(1);
		}
	}
	exit (0);
}


static
void uniconify(void)
{
	my_msg.cmd = CMD_UNICONIFY;
	if (!send_cmd()) exit (-1);
}


static
int send_cmd(void)
{
	int sock_fd;
	struct sockaddr_un sock_addr;
        struct timeval t;
	fd_set wrfds;
	int o, s;
	gchar sock_name[LOCAL_SOCK_SIZE];

	snprintf (sock_name, sizeof(sock_name),
		  "%s%s",
		  g_get_home_dir(),
		  WORK_DIR
		);

	unless (access(sock_name, R_OK|W_OK|X_OK)) { /* socket in home dir */

		snprintf (sock_name, sizeof(sock_name),
			  "%s%s/%s",
			  g_get_home_dir(),
			  WORK_DIR,
			  SOCKET_NAME
			);
	} else { /* system wide */

		snprintf (sock_name, sizeof(sock_name),
			  "%s/%s",
			  TMP_DIR,
			  SOCKET_NAME
			);
	}

	sock_addr.sun_family = AF_UNIX;
	strncpy(sock_addr.sun_path, sock_name, sizeof (sock_addr.sun_path));

	if ((sock_fd = socket(AF_UNIX, SOCK_DGRAM, 0)) == -1) {
                LOG("socket() error: %s", strerror (errno));
                return FALSE;
        }

	o = fcntl (sock_fd ,F_GETFL);
        if (o < 0) {
                LOG("fcntl() error: %s", strerror(errno));
                return FALSE;
         }
         o = (o|O_NONBLOCK);
         if (fcntl(sock_fd,F_SETFL,o) < 0) {
                LOG("fcntl()error: %s", strerror(errno));
                return FALSE;
         }      

	t.tv_sec = 0; t.tv_usec = 500000;
	FD_ZERO(&wrfds);
        FD_SET(sock_fd, &wrfds);
	
	if (select(sock_fd + 1, NULL, &wrfds, NULL, &t)==-1) {
                LOG("select() error: %s", strerror(errno));
                return FALSE;
        }

        if ((!FD_ISSET(sock_fd, &wrfds))) {
                LOG("unable write()");
                return FALSE;
        }

        if ((s = sendto(sock_fd, &my_msg, sizeof(my_msg), 0, &sock_addr, sizeof(struct sockaddr_un))) == -1) {
                LOG("sendto() error: %s", strerror(errno));
                return FALSE;
        } else {
                LOG("%d sent", s);
        }
        return TRUE;
}


static
void print_about(void)
{
	printf("%s (%s, %s). Was written by (c) Alexey Semenov 2001-2003.\n", GTKDICCTL_NAME, PACKAGE, VERSION);
	printf("Usage: %s -c command, available commands are:\n",GTKDICCTL_NAME);
	printf("       %s\n",S_CMD_CLIPBOARD_SEARCH_WORD);
	printf("       %s\n",S_CMD_CLIPBOARD_SEARCH_ART);
	printf("       %s\n",S_CMD_ICONIFY);
	printf("       %s\n",S_CMD_UNICONIFY);
	printf("       %s\n",S_CMD_QUIT_NO_SAVE);
}



/* __END__ */
