/* $Id: logging.h,v 1.9 2003/01/24 20:29:24 swaj Exp $ */

/*
 * Was written by Alexey Semenov (c) 2001-2003
 * See COPYING for copying policy
 */


#ifndef __LOGGING_H_
#define __LOGGING_H_ 1

#include <stdio.h>

#ifndef _LOG_TARGET_
#define _LOG_TARGET_ stderr
#endif

#ifndef DEBUG

#if defined (__STDC_VERSION__) && !defined(OLD_GNUC)
#define LOG(...)
#elif defined (__GNUC__)
#define LOG(format...)
#else  /* ! __GNUC__ */

#ifndef __LOG__
#define __LOG__ 1
	static void LOG(const char *format, ...) { }
#endif /* __LOG__ */

#endif /* __GNUC__ */

#else /* DEBUG  */


#ifndef __LOGGING_WITH_TIME

#define  __LOGGING_WITH_TIME 1

#include <time.h>
#include <sys/time.h>
#include <unistd.h>
#include <string.h>


static
char *  __attribute((unused)) __logging_safe_strtok(char *s, char pa)
{
	static char *p, *p1;

	if (s) { /* first entry */
		p = s;
		for (;;) {
			if (!*p) return s;
			if (*p == pa) {
 				*p++ = 0x0;
				for (;;) {
					if (!*p) return s;
					if (*p != pa) return s;
					*p++ = 0x0;
				}
				return s;
			}
			p++;
		}

	} else { /* next entries */

		p1 = p;
		for (;;) {
			if (!*p) return p1;
			if (*p == pa) {
 				*p++ = 0x0;
				for (;;) {
					if (!*p) return p1;
					if (*p != pa) return p1;
					*p++ = 0x0;
				}
				return p1;
			}
			p++;
		}
	}
}


static 
char * __attribute((unused)) __logging_time_string(void)
{
	struct timeval timeval;
	static char time_string[32];
        char  *ctt=NULL, *day=NULL, *month=NULL, *hour=NULL;

        gettimeofday(&timeval, NULL);
        ctt = ctime(&timeval.tv_sec);

        __logging_safe_strtok(ctt,' ');
	month = __logging_safe_strtok(NULL,' ');
	day = __logging_safe_strtok(NULL,' ');
	hour = __logging_safe_strtok(NULL,' ');

        snprintf(time_string,
		 sizeof (time_string),
		 "%s %s %s.%6.6d",
		 day,
		 month,
		 hour,
		 (int)timeval.tv_usec
		);

        return time_string;
}

#define __T __logging_time_string()

#endif /* __LOGGING_WITH_TIME */



#if defined (__STDC_VERSION__) && !defined(OLD_GNUC)


#ifdef HAVE_GLIB_H

#define LOG(...) { \
	__T; \
	g_print ("%s >> file %s: line %d (%s):", __T, __FILE__,__LINE__,__PRETTY_FUNCTION__ ); \
	g_log (G_LOG_DOMAIN, G_LOG_LEVEL_MESSAGE, __VA_ARGS__ ); }


#else /* !HAVE_GLIB_H */



#define LOG(...) \
	fprintf (_LOG_TARGET_, "%s >> file %s: line %d (%s):", __T, __FILE__,__LINE__,__PRETTY_FUNCTION__ ); \
	fprintf (_LOG_TARGET_, " Message: "); \
	fprintf (_LOG_TARGET_, __VA_ARGS__ ); \
	fprintf (_LOG_TARGET_, "\n");
#endif /* HAVE_GLIB_H */

#elif defined (__GNUC__)

#ifdef HAVE_GLIB_H
#define LOG(format...) \
	g_print ("%s >> file %s: line %d: ", __T, __FILE__,__LINE__ ); \
	g_log (G_LOG_DOMAIN, G_LOG_LEVEL_MESSAGE, format );
#else  /* !HAVE_GLIB_H */
#define LOG(format...) \
	fprintf (_LOG_TARGET_, "%s >> file %s: line %d: ", __T, __FILE__,__LINE__); \
	fprintf (_LOG_TARGET_, " Message: "); \
	fprintf (_LOG_TARGET_, format); \
	fprintf (_LOG_TARGET_, "\n");
#endif /* HAVE_GLIB_H */

#else /* ! __GNUC__ */

#ifndef __LOG__
#define __LOG__ 1

static void LOG(const char *format, ...)
{
	va_list args;
	fprintf (_LOG_TARGET_, "%s >> file %s: line %d ", __T, __FILE__,__LINE__);
	fprintf (_LOG_TARGET_,"log: ");
	va_start(args, format);
	fprintf (_LOG_TARGET_,format);
	va_end(args);
	fprintf (_LOG_TARGET_,"\n");
}
#endif /* __LOG__ */

#endif /* __GNUC__ */

#endif /* DEBUG */



#endif /* __LOGGING_H_ */



/* __END__ */

