/* $Id: util.c,v 1.13 2003/01/24 18:47:21 swaj Exp $ */

/*
 * Was written by Alexey Semenov (c) 2001-2003
 * See COPYING for copying policy
 */


#define __UTIL_C_
#include "util.h"
#undef __UTIL_C_


#include "db_sql.h"


void ut_mask_sigsegv (void)
{

#ifdef HAVE_SIGNAL_H
	signal (SIGSEGV, ut_sigsegv_handler);
	LOG("assign sigsegv handler");
#else
	LOG("WARNING: seems no HAVE_SIGNAL_H defined");

#endif /* HAVE_SIGNAL_H */
}


void ut_sigsegv_handler (gint signo)
{
#ifdef HAVE_SIGNAL_H

#ifdef HAVE_SYSLOG_H
	syslog(LOG_WARNING, "got SIGSEGV, please report this bug!");
#endif /* HAVE_SYSLOG_H */

#ifdef DEBUG
	LOG("got SIGSEGV, please report this bug!");
	db_shut();
	exit (-1);
#else
	fprintf (stderr, "got SIGSEGV, please report this bug!");
	db_shut();
	exit (-1);
#endif /* DEBUG */

#endif /* HAVE_SIGNAL_H */
}


gint ut_is_word_latin (gchar *word)
{
	gint a;	unless (word) {
		LOG("word = null");
		return TRUE;
	}


	unless (word) {
		LOG("word = null");
		return TRUE;
	}
/* latin if at least 1 ascii char */
	for (a=0; a < strlen(word); a++) {
		if ( ((word[a] >= 0x41) && (word[a] <= 0x5a)) ||
		     ((word[a] >= 0x61) && (word[a] <= 0x7a)) ) {
			return TRUE;
		}
	}
	return FALSE;
}


gint ut_is_word_bold (gchar *word)
{
/*
 *	gint a;
 */

	unless (word) {
		LOG("word = null");
		return FALSE;
	}

/* if ':' in word consider it bold */

	if (strchr(word, ':')) return TRUE;

/*
 *	for (a=0; a < strlen(word); a++) {
 *		if (word[a] == ':') {
 *			return TRUE;
 *		}
 *	 }
 */

	return FALSE;
}


void ut_umltotxt (gchar *word)
{
	gchar* ts;
	gchar c;
	gint a, b = 0;

	unless (word) {
		LOG("word = null");
		return;
	}

	ts = g_strdup(word);
	
	for (a=0; a < strlen(ts); a++) {

/* we track "\x" or "x:" combinations */ 
		c = ts[a];
		if ((c == '\\') && (a == (strlen(ts)-1))) {
			break;
		}
		if ((c == ':') && (!a)) {
			LOG("illegal ':' at begin of line");
			continue;
		}

		if ((c == '\\')||(c == ':')) { 

			if (c == '\\') {
				a++;
				c = ts[a];
			}

			if (c == ':') {
				c = ts[a-1];
				b--;
			}

			switch (c) {
			case 'a':
				c = 0xe4;
				break;
			case 'A':
				c = 0xc4;
				break;
			case 'o':
				c = 0xf6;
				break;
			case 'O':
				c = 0xd6;
				break;
			case 'e':
				c = 0xeb;
				break;
			case 'E':
				c = 0xcb;
				break;
			case 'u':
				c = 0xfc;
				break;
			case 'U':
				c = 0xdc;
				break;
			default:
				break;
			}
		}			

		word[b] = c;
		b++;
	}
	word[b] = 0x0;
	g_free(ts);
	return;
}


/*
 * completly free list ( with free(pointers) )
 */
void ut_slist_free (GSList* list)
{
	GSList* tmp = NULL;

	tmp = list;
	while (tmp != NULL) {
		g_free(GINT_TO_POINTER(tmp->data));
		tmp = g_slist_next(tmp);
	}
	g_slist_free(list);
	g_slist_free(tmp);
}


void ut_init_history (void)
{
	gint a; 

	for (a=0; a < HIST_LEN; a++ ) {
		word_history [a] = g_malloc (HIST_ITEM_LEN);
		memset (word_history[a], 0, HIST_ITEM_LEN);
	}

	for (a=0; a < HIST_LEN; a++ ) {
		art_history [a] = g_malloc (HIST_ITEM_LEN);
		memset (art_history[a], 0, HIST_ITEM_LEN);
	}
	word_history_ptr = 0;
	art_history_ptr = 0;
	word_history_len = 0;
	art_history_len = 0;
}


void ut_insert_history_word (gchar *word)
{
	LOG("inserting %s into word history", word);
	ut_insert_history(word, word_history, &word_history_len);
}


void ut_insert_history_art (gchar *word)
{
	LOG("inserting %s into word history", word);
	ut_insert_history(word, art_history, &art_history_len);
}


static
void ut_insert_history (gchar *word, gchar **history, gint *calc_len)
{
	gchar hist_tmp [HIST_LEN][HIST_ITEM_LEN];
	gint a=0, b=0, len = 0;

	unless (word) {
		LOG("word = null");
		return;
	}
	unless (history) {
		LOG("his4tory = null");
		return;
	}
	unless (calc_len) {
		LOG("calc_len = null");
		return;
	}

	for (a=0; a<HIST_LEN; a++) {
		memset(hist_tmp[a], 0, HIST_ITEM_LEN); 
	} 

	strncpy (hist_tmp[0], word,HIST_ITEM_LEN-1);

	for (a = 1; a < HIST_LEN; a++) {
		if (strcmp (word, history[b]) != 0) {
			strncpy (hist_tmp[a], history[b], HIST_ITEM_LEN-1);
		}
		b++;
	}
	for (a=0; a < HIST_LEN; a++) {
		strncpy (history[a], hist_tmp[a], HIST_ITEM_LEN-1);
		if (strlen(history[a]) > 0) {
			len++;
		}
	}
	*calc_len = len;
	LOG("len = %d", *calc_len);
}


gchar* ut_fetch_history_word_up (void)
{
	gchar *p;
	LOG("fetch from word history");
	if (!word_history_len) {
		LOG("history empty");
		return NULL;
	}
	p = ut_fetch_history (word_history, &word_history_ptr);

	if ((p!=NULL) && (++word_history_ptr >= word_history_len)) {
		word_history_ptr=0;
	}
	if (strlen(p) == 0) {
		return NULL;
	} else {
		return p;
	}
}


gchar* ut_fetch_history_word_down (void)
{
	gchar *p;
	LOG("fetch from word history");
	if (!word_history_len) {
		LOG("history empty");
		return NULL;
	}
	p = ut_fetch_history (word_history, &word_history_ptr);

	if ((p!=NULL) && (--word_history_ptr < 0)) {
		word_history_ptr = word_history_len-1;
	}

	if (strlen(p) == 0) {
		return NULL;
	} else {
		return p;
	}
}


gchar* ut_fetch_history_art_up (void)
{
	gchar *p;
	LOG("fetch from art history");
	if (!art_history_len) {
		LOG("history empty");
		return NULL;
	}
	p = ut_fetch_history (art_history, &art_history_ptr);

	if ((p!=NULL) && (++art_history_ptr >= art_history_len)) {
		art_history_ptr=0;
	}
	if (strlen(p) == 0) {
		return NULL;
	} else {
		return p;
	}
}


gchar* ut_fetch_history_art_down (void)
{
	gchar *p;
	LOG("fetch from art history");
	if (!art_history_len) {
		LOG("history empty");
		return NULL;
	}
	p = ut_fetch_history (art_history, &art_history_ptr);

	if ((p!=NULL) && (--art_history_ptr < 0)) {
		art_history_ptr = art_history_len-1;
	}
	if (strlen(p) == 0) {
		return NULL;
	} else {
		return p;
	}
}


static
gchar* ut_fetch_history (gchar **history, gint *ptr)
{
	LOG("fetched %s", history[*ptr]); 
	return history[*ptr];
}


/*
 * for LaTeX output
 */
gchar* ut_add_uml (gchar *word)
{
	static gchar temp_str[WORD_MAX_LEN];
	gint a;
	guchar ch;

	memset (temp_str, 0, sizeof(temp_str));

	if (strlen(word)*3 >= sizeof(temp_str)) {
		LOG("long word");
		return temp_str;
	}

	for (a = 0; a < strlen(word); a++) {
		ch = word[a];

		switch (ch) {
			/* TODO other iso-8859-1 letters */
		case 0xe4:
			g_snprintf(temp_str + strlen(temp_str), sizeof(temp_str) - strlen(temp_str), "\\\"a");
			break;
		case 0xc4:
			g_snprintf(temp_str + strlen(temp_str), sizeof(temp_str) - strlen(temp_str), "\\\"A");
			break;
		case 0xf6:
			g_snprintf(temp_str + strlen(temp_str), sizeof(temp_str) - strlen(temp_str), "\\\"o");
			break;
		case 0xd6:
			g_snprintf(temp_str + strlen(temp_str), sizeof(temp_str) - strlen(temp_str), "\\\"O");
			break;
		case 0xfc:
			g_snprintf(temp_str + strlen(temp_str), sizeof(temp_str) - strlen(temp_str), "\\\"u");
			break;
		case 0xdc:
			g_snprintf(temp_str + strlen(temp_str), sizeof(temp_str) - strlen(temp_str), "\\\"U");
			break;
		case 0xeb:
			g_snprintf(temp_str + strlen(temp_str), sizeof(temp_str) - strlen(temp_str), "\\\"e");
			break;
		case 0xcb:
			g_snprintf(temp_str + strlen(temp_str), sizeof(temp_str) - strlen(temp_str), "\\\"E");
			break;
		default:
			g_snprintf(temp_str + strlen(temp_str), sizeof(temp_str) - strlen(temp_str), "%c", ch);
		}	
	}
	return temp_str;
}


gint ut_is_line_empty (gchar* s)
{
	gint a;
	gchar c;

	if (s == NULL) {
		LOG("s = null");
		return TRUE;
	}

	for (a=0; a<strlen(s); a++) {
		c = s[a];
		switch (c) {
		case ' ':
		case '\t':
		case '\n':
		case '\r':
			continue;
		default:
			return FALSE;
		}
	}
	return TRUE;
}


void ut_replace_control (unsigned char *str)
{
	gint a;

	LOG("now here");

	if ((str == NULL) || (strlen(str) ==0)) {
		LOG("invalid str");
		return;
	}

	for (a=0; a<strlen(str); a++) {
		if (str[a] == CR) {
			str[a] = CR_REPL;
			continue;
		}

		if ( (str[a] == TAB) ||
		     (str[a] == NL)
		   ) {
			continue;
		}

		if ((str[a] < 0x20) || (str[a]==0 ))  {
			str[a] = CR_REPL;
			continue;
		}

	}
}



/* __END__ */
